# -*- coding: ISO-8859-1 -*-

############################################################################## 
## 2004, Arnulf Heimsbakk
## 
## THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
## AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
## IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
## ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR 
## ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
## DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
## SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
## CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
## LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
## OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
## SUCH DAMAGE.
##
## Otherwise the GPL license: http://www.gnu.org/licenses/gpl.txt
##############################################################################

import os, os.path, cPickle, time, EXIF, sys, framer, stat,effects, types, traceback

from cStringIO import StringIO
from gzip import GzipFile
from sha import sha
from md5 import md5
from PIL import Image, ImageFile

from framer import *

ImageFile.MAXBLOCK = 2**22 # set to 4Mb, default is 64k

class thumbnailer:
    def __init__(self, paths, size=(100,100), ftype="jpg", fquality=85, effect=None, framer=None):
        self._size = size                                   # thumbnail size
        self._cache_path, self._meta_path = paths           # chache path
        self._ftype = ftype                                 # thumbnail file type
        self._fquality = fquality                           # jpg quality
        self.framer = framer                                # framer used
        self.effect = effect
       
    def get(self, filename, uid='', append='', make_thumb=False):
        cfile = uid
        if append != '':
            cfile += '_%s' % append 
        cfile = os.path.join(self._cache_path, "%s.%s" % (cfile, self._ftype))

        if not os.path.exists(cfile) and make_thumb:
            thumb = self._make_thumb(filename)
            if thumb != None:
                    thumb.save(file(cfile, "wb"), 
                               optimize=1, 
                               quality=self._fquality)

        return cfile, os.stat(cfile)[stat.ST_SIZE]

    def flush(self):
        for i in os.listdir(self._cache_path):
            os.remove(os.path.join(self._cache_path, i))
        for i in os.listdir(self._meta_path):
            os.remove(os.path.join(self._meta_path, i))
        
    def resize(self, filename, size=(100,100)):
        cfile, img_size = self.get(filename)
        
        if meta != None:
            try:
                image = Image.open(filename)
                if image.mode != 'RGB':
                    image = image.convert('RGB')
                image = self._make_thumb(filename)
                images = StringIO()
                image.save(images, 
                           self._ftype, 
                           optimize=1,
                           quality=self._fquality)
                return images
            except: pass
        return None

    def convert(self, filename):
        image = Image.open(file(filename, 'rb'))
        converted = StringIO()
        
        if image.mode != 'RGB':
            image = image.convert('RGB')
        optimize = 0
        
        image.save(converted, 
                   self._ftype, 
                   optimize=optimize,
                   quality=self._fquality)
        return converted
        
    def is_image(self, filename):
        try:
            s = os.stat(filename)
            image = Image.open(file(filename, 'rb'))
            meta = {'IMAGE size': s[stat.ST_SIZE],
                    'IMAGE modified': s[stat.ST_MTIME],  
                    'IMAGE format': image.format,
                    'IMAGE dimension' : image.size,
                    'IMAGE colormode': image.mode,
                    'IMAGE info': image.info}
            if meta['IMAGE info'].has_key('exif'):
                meta['IMAGE exif'] = self._decode_exif(meta['IMAGE info']['exif'])
                del meta['IMAGE info']['exif'] 

            return meta
        except:
            return None
    
    def _make_thumb(self, fname):
        try:
            image = Image.open(file(fname, "rb"))
            if image.mode != 'RGB':
                image = image.convert('RGB')
            image.thumbnail(self._size, Image.ANTIALIAS)
            
            if self.effect != None:
                image = self.effect(image)
            if self.framer != None:
                image = self.framer(image)
            return image
        except:
            traceback.print_exc()
            return None
            
    def _decode_exif(self, exif):
        exif = StringIO(exif)
        exif.seek(6)
        offset = exif.tell()
        endian = exif.read(1)
        
        # Use Gene Cash's EXIF Library to extract EXIF info which PIL did find.
        # Below is modified code of the EXIF.process_file method.
        # In using it this way we do not need to open the image file in two
        # seperate operations to extract the information. A small speedup.
        hdr=EXIF.EXIF_header(exif, endian, offset, 0)
        ifd_list=hdr.list_IFDs()
        ctr=0
        for i in ifd_list:
            if ctr == 0:
                IFD_name='Image'
            elif ctr == 1:
                IFD_name='Thumbnail'
                thumb_ifd=i
            else:
                IFD_name='IFD %d' % ctr
            hdr.dump_IFD(i, IFD_name)
            exif_off=hdr.tags.get(IFD_name+' ExifOffset')
            if exif_off:
                hdr.dump_IFD(exif_off.values[0], 'EXIF')
                intr_off=hdr.tags.get('EXIF SubIFD InteroperabilityOffset')
                if intr_off:
                    hdr.dump_IFD(intr_off.values[0], 'EXIF Interoperability',
                                 dict=INTR_TAGS)
            gps_off=hdr.tags.get(IFD_name+' GPSInfo')
            if gps_off:
                hdr.dump_IFD(gps_off.values[0], 'GPS', dict=GPS_TAGS)
            ctr+=1
    
        if hdr.tags.has_key('EXIF MakerNote'):
            hdr.decode_maker_note()
        
        # Removes unknown and unwanted tags here
        tags = hdr.tags
        for i in tags.keys():
            # Making own thumnails, do not need embedded thumbnail info
            if i.find('Thumbnail') > -1:
                del tags[i]
            # Already have the decoded marker note tags    
            elif i == "EXIF MakerNote":
                del tags[i]
            # Removes unknown marker note tags
            elif i.find("MakerNote Tag 0x") > -1:
                del tags[i]
        
        # Convert tags from Gene Cach's object to dicts
        tmp_exif = {}
        for i in tags.keys():
            field_type = EXIF.FIELD_TYPES[tags[i].field_type][2]
            values = tags[i].values
            if field_type.find('Ratio') > -1:
                tmp = []
                for j in values: 
                    try: tmp.append(j.num / float(j.den))
                    except: tmp.append(j.num)
                values = tmp
            tmp_exif[i] = {'field_type': field_type, 
                           'value': values, 
                           'printable': tags[i].printable,
                           'tag': tags[i].tag,
                           'field_length': tags[i].field_length, 
                           'field_offset': tags[i].field_offset}
        return tmp_exif


if __name__ == '__main__':
    t = thumbnailer(("",""))
    exif = t.is_image("tt.JPG")['IMAGE exif']

    for i in exif:
        print i, exif[i]
